from .websocket import subscribe
from .websocket import get_max_ws_uri

import websockets
import asyncio
import json
from loguru import logger


class Stream(object):

    def __init__(self, parser) -> None:
        self.parser = parser

    def subscribe(self, subscription):
        pass

    async def handler(uri, messages):
        async with websockets.connect(uri) as websocket:
            for message in messages:
                await websocket.send(json.dumps(message))

            while True:
                try:
                    response = await websocket.recv()
                    logger.info(response)
                except websockets.ConnectionClosed as e:
                    logger.error(e)
                    break



from __future__ import annotations

import uuid
from dataclasses import dataclass
from enum import Enum
from typing import List


class Channel(Enum):
    BOOK = 'book'
    TRADE = 'trade'
    TICKER = 'ticker'
    USER = 'user'


@dataclass
class Subscription:
    channel: Channel
    market: str
    depth: int = None

    def to_dict(self) -> dict:
        d = {
            'channel': self.channel.value,
            'market': self.market,
        }

        if self.depth is not None:
            d['depth'] = self.depth

        return d

    @classmethod
    def parse(self, d: dict) -> Subscription:
        return Subscription(
            channel=Channel(d.get('channel')),
            market=d.get('market'),
            depth=d.get('depth'),
        )


@dataclass
class SubscriptionList:
    subscriptions: List[Subscription]

    def to_dict(self) -> dict:
        d = {
            'action': 'sub',
            'subscriptions': [s.to_dict() for s in self.subscriptions],
            'id': str(uuid.uuid4()),
        }
        return d



from __future__ import annotations

import hmac
import uuid
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List

from .utils import get_api_key_from_env, get_api_secret_from_env


class Filter(Enum):
    ORDER = 'order'
    TRADE = 'trade'
    ACCOUNT = 'account'
    TRADE_UPDATE = 'trade_update'


@dataclass
class Auth:
    api_key: str
    nonce: int
    signature: str
    filters: List[Filter] = None

    @classmethod
    def from_env(cls) -> Auth:
        api_key = get_api_key_from_env()
        api_secret = get_api_secret_from_env()

        nonce = int(datetime.now().timestamp() * 1000)

        signature = hmac.new(api_secret.encode(), digestmod='sha256')
        signature.update(str(nonce).encode())
        signature = signature.hexdigest()

        return cls(
            api_key=api_key,
            signature=signature,
            nonce=nonce,
        )

    def to_msg(self) -> dict:
        d = {
            'action': 'auth',
            'apiKey': self.api_key,
            'nonce': self.nonce,
            'signature': self.signature,
            'id': str(uuid.uuid4()),
        }

        if self.filters:
            d['filters'] = [f.value for f in self.filters]

        return d



from __future__ import annotations

from dataclasses import dataclass
from typing import List

from . import Channel
from . import Subscription

from enum import Enum


class Event(Enum):
    AUTHENTICATED = 'authenticated'
    SUBSCRIBED = 'subscribed'
    UNSUBSCRIBED = 'unsubscribed'
    SNAPSHOT = 'snapshot'
    ORDER_SNAPSHOT = 'order_snapshot'
    TRADE_SNAPSHOT = 'trade_snapshot'
    ACCOUNT_SNAPSHOT = 'account_snapshot'
    ERROR = 'error'
    UPDATE = 'update'


@dataclass
class AuthenticatedEvent:
    event: Event
    id: str
    at: int

    @classmethod
    def parse(cls, d: dict) -> AuthenticatedEvent:
        event = Event(d.get('e'))
        id = d.get('i')
        at = d.get('T')

        return cls(event, id, at)


@dataclass
class SubscribedEvent:
    event: Event
    subscriptions: List[Subscription]
    id: str
    at: str

    @classmethod
    def parse(cls, d: dict) -> SubscribedEvent:
        event = Event(d.get('e'))
        subscriptions = [Subscription.parse(s) for s in d.get('s')]
        id = d.get('i')
        at = d.get('T')

        return cls(event, subscriptions, id, at)


@dataclass
class UnsubscribedEvent:
    event: Event
    subscriptions: List[Subscription]
    id: str
    at: str

    @classmethod
    def parse(cls, d: dict) -> UnsubscribedEvent:
        event = Event(d.get('e'))
        subscriptions = [Subscription.parse(s) for s in d.get('s')]
        id = d.get('i')
        at = d.get('T')

        return cls(event, subscriptions, id, at)


@dataclass
class PriceVolume:
    price: float
    volume: float

    @classmethod
    def parse(cls, asks_or_bids: list) -> List[PriceVolume]:
        return [PriceVolume(float(p), float(v)) for p, v in asks_or_bids]


# {
#  "c": "book",
#  "e": "snapshot",
#  "M": "btcusdt",
#  "a": [["5337.3", "0.1"]],
#  "b": [["5333.3", "0.5"]],
#  "T": 1591869939634
# }
@dataclass
class BookSnapshot:
    channel: Channel
    event: Event
    market: str
    asks: List[PriceVolume]
    bids: List[PriceVolume]
    at: str

    @classmethod
    def parse(cls, d: dict) -> BookSnapshot:
        channel = Channel(d.get('c'))
        event = Event(d.get('e'))
        market = d.get('M')
        asks = PriceVolume.parse(d.get('a'))
        bids = PriceVolume.parse(d.get('b'))
        at = d.get('T')

        return cls(channel, event, market, asks, bids, at)


def parse_snapshot(d: dict):
    channel = Channel(d.get('c'))

    if channel == Channel.BOOK:
        return BookSnapshot.parse(d)
    elif channel == Channel.TICKER:
        pass


def parse_response(d: dict):
    event = Event(d.get('e'))
    if event == Event.AUTHENTICATED:
        return AuthenticatedEvent.parse(d)
    elif event == Event.SUBSCRIBED:
        return SubscribedEvent.parse(d)
    elif event == Event.SUBSCRIBED:
        return UnsubscribedEvent.parse(d)
    elif event == event.SNAPSHOT:
        return parse_snapshot(d)



import asyncio
import json

import websockets
from loguru import logger

from . import Auth
from . import Channel
from . import Subscription
from . import SubscriptionList
from .events import parse_response
from .utils import get_max_ws_uri


async def subscribe(uri, messages):
    async with websockets.connect(uri) as websocket:
        for message in messages:
            await websocket.send(json.dumps(message))

        while True:
            try:
                response = await websocket.recv()
                event = parse_response(json.loads(response))
                if event is not None:
                    logger.info(event)

            except websockets.ConnectionClosed as e:
                logger.error(e)
                break


def main():
    messages = [
        Auth.from_env().to_msg(),
        SubscriptionList([
            Subscription(Channel.BOOK, 'btcusdt', depth=1),
            Subscription(Channel.TRADE, 'btcusdt'),
            Subscription(Channel.TICKER, 'btcusdt'),
            Subscription(Channel.USER, 'btcusdt'),
        ]).to_dict(),
    ]

    uri = get_max_ws_uri()

    asyncio.run(subscribe(uri, messages))


if __name__ == '__main__':
    main()



import os

MAX_WS_URI = 'wss://max-stream.maicoin.com/ws'


def get_max_ws_uri():
    max_ws_uri = os.environ.get('MAX_WS_URI')

    if max_ws_uri is None:
        max_ws_uri = MAX_WS_URI

    return max_ws_uri


def get_api_key_from_env():
    return os.environ.get('MAX_API_KEY')


def get_api_secret_from_env():
    return os.environ.get('MAX_API_SECRET')
